-- 543 latched xcvr with oe
library ieee;
use ieee.std_logic_1164.all;
library exemplar;
use exemplar.exemplar_1164.all;

ENTITY pulse_widths IS
    GENERIC (
        tlow: TIME := 0 ns;
        thigh: TIME := 0 ns);
    PORT (
        in0 : IN std_logic;
        out0: OUT std_logic);
END pulse_widths;

library ieee;
use ieee.std_logic_1164.all;
library exemplar;
use exemplar.exemplar_1164.all;

ARCHITECTURE behavior OF pulse_widths IS
BEGIN
    PROCESS (in0)
        VARIABLE prev_event : TIME := 0 ns;
    BEGIN
        IF prev_event /= 0 ns THEN
            IF in0 = '1' THEN
                ASSERT Now - prev_event > tlow
                    REPORT "Pulse width of '0' is less than " & time'image(tlow)
                    SEVERITY Warning;
            ELSIF in0 = '0' THEN
                ASSERT Now - prev_event > thigh
                    REPORT "Pulse width of '1' is less than " & time'image(thigh)
                    SEVERITY Warning;
            END IF;
        END IF;

        prev_event := Now;
    END PROCESS;

    out0 <= in0;
END behavior;

library IEEE;
use IEEE.std_logic_1164.all;
library exemplar;
use exemplar.exemplar_1164.all;

entity latchxcvr is
generic ( 
        tDY :time := 8.5 ns; tEN : time := 12 ns; tDIS : time := 9 ns; tPLO : time := 5 ns; tPHI : time := 7 ns;
          tDS : time := 3 ns; tDH : time := 2 ns);
    port (
		not_leab,not_oeab,not_ceab : in std_logic;
		not_leba,not_oeba,not_ceba : in std_logic;
        a : inout STD_LOGIC;
        b : inout STD_LOGIC
    );
end latchxcvr;

library IEEE;
use IEEE.std_logic_1164.all;
library exemplar;
use exemplar.exemplar_1164.all;

architecture latchxcvr_arch of latchxcvr is
signal data_a,data_b : std_logic;
COMPONENT pulse_widths
    GENERIC (
        tlow: TIME := 0 ns;
        thigh: TIME := 0 ns);
    PORT (
        in0 : IN std_logic;
        out0: OUT std_logic);
END COMPONENT;

SIGNAL  tnot_leab,tnot_leba : Std_logic; -- temps for pulse width checks
begin
-- not_leab to a setup and hold checks
l : ASSERT not((not_leab='1' and not not_leab'stable) and (not a'stable(tDS))) 
        REPORT "setup violation not_leab to a"
        SEVERITY warning;

m : ASSERT not((not_leab='1' and a'event) and (not not_leab'stable(tDH))) 
        REPORT "hold violation not_leab to a"
        SEVERITY warning;
-- not_leba to a setup and hold checks
n : ASSERT not((not_leba='1' and not not_leba'stable) and (not b'stable(tDS))) 
        REPORT "setup violation not_leba to b"
        SEVERITY warning;

o : ASSERT not((not_leba='1' and b'event) and (not not_leba'stable(tDH))) 
        REPORT "hold violation not_leba to b"
        SEVERITY warning;


-- pulse width minimums tests
pnot_leab: pulse_widths 
    GENERIC MAP (tlow => tPLO, thigh => tPHI)
    PORT MAP (in0 => not_leab, out0 => tnot_leab);
pnot_leba: pulse_widths 
    GENERIC MAP (tlow => tPLO, thigh => tPHI)
    PORT MAP (in0 => not_leba, out0 => tnot_leba);


proc_eab: process (not_leab,not_ceab,not_oeab,a,data_a)
begin
if (not_oeab='0') then
  if ((not_leab='0') and (not_ceab='0')) then
    data_a <= a;
	b <= a	after tDY;
  else 
    b <= data_a after tEN;
  end if;
else
  if ((not_leab='0') and (not_ceab='0')) then
    data_a <= a;
  end if;
  b <= 'Z' after tDIS;
end if;
end process proc_eab;

proc_eba: process (not_leba,not_ceba,not_oeba,b,data_b)
begin
if (not_oeba='0') then
  if ((not_leba='0') and (not_ceba='0')) then
    data_b <= b;
	a <= b	after tDY;
  else 
    a <= data_b after tEN;
  end if;
else
  if ((not_leba='0') and (not_ceba='0')) then
    data_b <= b;
  end if;
  a <= 'Z' after tDIS;
end if;
end process proc_eba;

end latchxcvr_arch;

library IEEE;
use IEEE.std_logic_1164.all;
library exemplar;
use exemplar.exemplar_1164.all;


entity latchxcvrn is
generic ( size :integer := 32;
        tDY :time := 8.5 ns; tEN : time := 12 ns; tDIS : time := 9 ns; tPLO : time := 5 ns; tPHI : time := 7 ns;
          tDS : time := 3 ns; tDH : time := 2 ns);
    port (
		not_leab,not_oeab,not_ceab : in std_logic;
		not_leba,not_oeba,not_ceba : in std_logic;
        a : inout STD_LOGIC_vector(size-1 downto 0);
        b : inout STD_LOGIC_vector(size-1 downto 0)
    );
end latchxcvrn;

library IEEE;
use IEEE.std_logic_1164.all;
library exemplar;
use exemplar.exemplar_1164.all;

architecture latchxcvrn_arch of latchxcvrn is
-- Component instantiated by Turbo autoplace on 01/08/96 at 15:05:20
COMPONENT latchxcvr
generic ( tDY :time := 8.5 ns; tEN : time := 12 ns; tDIS : time := 9 ns; tPLO : time := 5 ns; tPHI : time := 7 ns;
          tDS : time := 3 ns; tDH : time := 2 ns);
    port (
		not_leab,not_oeab,not_ceab : in std_logic;
		not_leba,not_oeba,not_ceba : in std_logic;
        a : inout STD_LOGIC;
        b : inout STD_LOGIC
    );
END COMPONENT;
begin

c_call: FOR i IN 0 TO size-1 GENERATE
          bits: latchxcvr
          generic map (tDY => tDY, tDIS=>tDIS, tEN=>tEN, tPLO=>tPLO, tPHI=>tPHI,tDS=>tDS,tDH=>tDH) 
          port map (
           not_leab=>not_leab,not_ceab=>not_ceab,not_oeab=>not_oeab,
           not_leba=>not_leba,not_ceba=>not_ceba,not_oeba=>not_oeba,
		   a=>a(i),b=>b(i)
           );
        END GENERATE;    
end latchxcvrN_arch;
